@@ -304,8 +304,6 @@ gkm_about_dialog_new (GtkWindow *transient_parent)
 {
   GtkWidget *about;
 
-  g_return_val_if_fail (GTK_IS_WINDOW (transient_parent), NULL);
-
   const char *authors[] = {
     "Fernando Herrera <fherrera@onirica.com>",
     "Mariano Su\303\241rez-Alvarez <mariano@gnome.org>",
@@ -313,9 +311,11 @@ gkm_about_dialog_new (GtkWindow *transient_parent)
     NULL
   };
 
-  const char *documenters[] = {
+  g_return_val_if_fail (GTK_IS_WINDOW (transient_parent), NULL);
+
+  /*  const char *documenters[] = {
     NULL
-  };
+    }; */
 
   gtk_about_dialog_set_email_hook ((GtkAboutDialogActivateLinkFunc) gkm_about_dialog_activate_links,
 				   GINT_TO_POINTER (LINK_TYPE_EMAIL), NULL);
@@ -325,19 +325,19 @@ gkm_about_dialog_new (GtkWindow *transient_parent)
 
   about = gtk_about_dialog_new ();
 
-  g_object_set (G_OBJECT (about),
-		"authors", authors,
-		"comments",  _("A simple keyring manager for GNOME 2"),
-		"copyright", _("Copyright \302\251 2004, 2005 The GNOME Love Project"),
-		"documenters", documenters, 
-		"license", GPL,
-		"logo-icon-name", "stock_keyring",
-		"name", _("GNOME Keyring Manager"),
-		"translator-credits", _("translator-credits"),
-		"version", VERSION,
-		"website", "http://www.gnome.org",
-		NULL);
-              
+  gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (about), authors);
+  gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (about), _("A simple keyring manager "
+							     "for GNOME 2"));
+  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (about), _("Copyright \302\251 2004, "
+							      "2005 The GNOME Love Project"));
+  /*  gtk_about_dialog_set_documenters (GTK_ABOUT_DIALOG (about), documenters); */
+  gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (about), GPL);
+  gtk_about_dialog_set_logo_icon_name (GTK_ABOUT_DIALOG (about), "stock_keyring");
+  gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (about), _("GNOME Keyring Manager"));
+  gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG (about), _("translator-credits"));
+  gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (about), VERSION);
+  gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (about), "http://www.gnome.org");
+
   gtk_window_set_transient_for (GTK_WINDOW (about), transient_parent);
   
   return about;
