@@ -152,10 +152,6 @@ gkm_application_window_destroyed_callback (GObject *window, gpointer user_data)
 int
 main (int argc, char **argv)
 {
-  if (setlocale (LC_ALL, "") == NULL)
-    {
-      g_warning ("locale not understood by C library, internationalization will not work.");
-    }
 
   bindtextdomain (GETTEXT_PACKAGE, GNOME_KEYRING_MANAGER_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
